package DiversityPackage;

import java.util.ArrayList;
import org.apache.jena.ontology.OntClass;
import org.apache.jena.ontology.OntModel;
import org.apache.jena.ontology.OntResource;
import org.apache.jena.query.QueryExecution;
import org.apache.jena.query.QueryExecutionFactory;
import org.apache.jena.query.QueryFactory;
import org.apache.jena.query.QuerySolution;
import org.apache.jena.query.QuerySolutionMap;
import org.apache.jena.query.ResultSet;
import org.apache.jena.rdf.model.Literal;
import org.apache.jena.rdf.model.ModelFactory;
import org.apache.jena.rdf.model.Resource;
import org.apache.jena.util.iterator.ExtendedIterator;

public class CountClassesAndInstances {

	public static int CountALLInstancesJena(String Source, String URI) {

		OntModel model = ModelFactory.createOntologyModel();
		model.read(Source, "RDF/XML");
		Resource r = model.getResource(URI);
		OntClass Concept = r.as(OntClass.class);
		ArrayList instances = new ArrayList();

		for (ExtendedIterator<? extends OntResource> i = Concept.listInstances(false); i.hasNext();) {

			OntResource ins = i.next();
			instances.add(ins);

		}
		return instances.size();

	}

	public static int CountALLClassesJena(String Source, String URI) {

		OntModel model = ModelFactory.createOntologyModel();
		model.read(Source, "RDF/XML");
		Resource r = model.getResource(URI);
		OntClass Concept = r.as(OntClass.class);

		ArrayList subclasses = new ArrayList();

		for (ExtendedIterator<OntClass> i = Concept.listSubClasses(false); i.hasNext();) {
			OntClass cls = i.next();
			subclasses.add(cls);

		}
		return subclasses.size();

	}

	public static int CountALLInstances(String Source, String URI) {
		int NoInstances = 0;

		OntModel model = ModelFactory.createOntologyModel();
		model.read(Source, "RDF/XML");
		Resource r = model.getResource(URI);
		OntClass Concept = r.as(OntClass.class);
		QuerySolutionMap initialSettings = new QuerySolutionMap();
		initialSettings.add("Concept", Concept);

		String queryString = "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>"
				+ "PREFIX owl: <http://www.w3.org/2002/07/owl#>"
				+ "PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>"
				+ "PREFIX xsd: <http://www.w3.org/2001/XMLSchema#>"
				+ "select  (count(?instance) as ?Number_of_All_Instances) "
				+ "WHERE{ ?subject rdfs:subClassOf* ?Concept." + "       ?instance a ?subject " + "}\n";

		org.apache.jena.query.Query query = QueryFactory.create(queryString);
		QueryExecution qe = QueryExecutionFactory.create(query, model, initialSettings);

		ResultSet resultSet = qe.execSelect();
		while (resultSet.hasNext()) {
			QuerySolution Solution = resultSet.next();
			Literal Instances = (Literal) Solution.get("Number_of_All_Instances");
			NoInstances = Instances.getInt();
		}

		qe.close();

		return NoInstances;

	}

	public static int CountSubclasses(String Source, String URI) {
		int NoSubclasses = 0;

		OntModel model = ModelFactory.createOntologyModel();
		model.read(Source, "RDF/XML");
		Resource r = model.getResource(URI);
		OntClass Concept = r.as(OntClass.class);

		QuerySolutionMap initialSettings = new QuerySolutionMap();
		initialSettings.add("Concept", Concept);

		String queryString = "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>"
				+ "PREFIX owl: <http://www.w3.org/2002/07/owl#>"
				+ "PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>"
				+ "PREFIX xsd: <http://www.w3.org/2001/XMLSchema#>"
				+ "select  (count(?subject) as ?Number_of_Subclasses) " + " WHERE { ?subject rdfs:subClassOf* ?Concept "
				+ "}\n";
		org.apache.jena.query.Query query = QueryFactory.create(queryString);
		QueryExecution qe = QueryExecutionFactory.create(query, model, initialSettings);

		ResultSet resultSet = qe.execSelect();

		while (resultSet.hasNext()) {
			QuerySolution Solution = resultSet.next();
			Literal Subclasses = (Literal) Solution.get("Number_of_Subclasses");
			NoSubclasses = Subclasses.getInt();
		}

		qe.close();

		return NoSubclasses;
	}
}
