
package DiversityPackage;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import org.apache.jena.ontology.Individual;
import org.apache.jena.ontology.OntClass;
import org.apache.jena.ontology.OntModel;
import org.apache.jena.ontology.OntResource;
import org.apache.jena.rdf.model.ModelFactory;
import org.apache.jena.rdf.model.Resource;
import org.apache.jena.util.iterator.ExtendedIterator;
import org.openrdf.model.URI;
import slib.graph.algo.utils.GAction;
import slib.graph.algo.utils.GActionType;
import slib.graph.io.conf.GDataConf;
import slib.graph.io.conf.GraphConf;
import slib.graph.io.loader.GraphLoaderGeneric;
import slib.graph.io.util.GFormat;
import slib.graph.model.graph.G;
import slib.graph.model.impl.graph.memory.GraphMemory;
import slib.graph.model.impl.repo.URIFactoryMemory;
import slib.graph.model.repo.URIFactory;
import slib.sml.sm.core.engine.SM_Engine;
import slib.sml.sm.core.utils.SMConstants;
import slib.sml.sm.core.utils.SMconf;
import slib.utils.ex.SLIB_Exception;

public class ShortestPathFromSML {

	URIFactory factory = URIFactoryMemory.getSingleton();
	final String ONTO_FILE = "PresentationOntologyV4.2Validated.owl";

	public Hashtable<Object, Float> computeShortestPath(Hashtable<String, ArrayList> EntitiesPerSuperClass,
			String source) throws SLIB_Exception {
		double Disparity = 0;
		// prepare for SML semantic distances calculations
		URI graphURI = factory.getURI(source);
		G g = new GraphMemory(graphURI);
		GDataConf dataConf = new GDataConf(GFormat.RDF_XML, ONTO_FILE);
		GAction actionRerootConf = new GAction(GActionType.REROOTING);
		GraphConf gConf = new GraphConf();
		gConf.addGDataConf(dataConf);
		gConf.addGAction(actionRerootConf);
		GraphLoaderGeneric.load(gConf, g);
		SMconf smConf = new SMconf(SMConstants.FLAG_SIM_PAIRWISE_DAG_EDGE_RADA_1989);
		SM_Engine engine = new SM_Engine(g);
		OntModel model = ModelFactory.createOntologyModel();
		model.read(source, "RDF/XML");

		// Access each super class and it is entities
		Hashtable<Object, Float> superClassDispersion = new Hashtable<Object, Float>();
		Enumeration en = EntitiesPerSuperClass.keys();
		while (en.hasMoreElements()) {

			Object Superclasses = en.nextElement();
			ArrayList EntitiesofSuperClasses = EntitiesPerSuperClass.get(Superclasses);

			List<List<Integer>> pathMatrix = new ArrayList<List<Integer>>(EntitiesofSuperClasses.size());
			Hashtable<Object, List<Integer>> PathMatrix = new Hashtable<Object, List<Integer>>(
					EntitiesofSuperClasses.size());

			for (int i = 0; i < EntitiesofSuperClasses.size(); i++) {

				int path = 0;
				List<Integer> pathLenght = new ArrayList<Integer>();
				Object u1 = EntitiesofSuperClasses.get(i);
				Object temp = u1;
				int pathCounter1 = 0;

				for (int j = 0; j < EntitiesofSuperClasses.size(); j++) {
					Object u2 = EntitiesofSuperClasses.get(j);
					int pathCounter2 = 0;
					if (u1.equals(u2)) {

						path = 0;
					} else {
						// u1 is an instance
						if (model.getIndividual(u1.toString()).isIndividual()) {

							u1 = FindParentClass(u1, model, Superclasses);
							pathCounter1++;
						}

						// c2 is an instance
						if (model.getIndividual(u2.toString()).isIndividual()) {
							u2 = FindParentClass(u2, model, Superclasses);
							pathCounter2++;
						}

						double score = engine.compare(smConf, factory.getURI(u1.toString()),
								factory.getURI(u2.toString()));
						path = ((int) (1 / score)) - 1;
						if (pathCounter1 != 0 || pathCounter2 != 0) {
							path = path + pathCounter1 + pathCounter2;
						}

					}
					pathLenght.add(path);
					pathCounter1 = 0;
					u1 = temp;
				}

				// Add each entity with its distances from all other entities
				PathMatrix.put(u1, pathLenght);
				// add the list of short paths to the overall list
				pathMatrix.add(pathLenght);

			}
			// calculate (dispersion) Ball-Hall Internal clustering validation for each
			// super class
			superClassDispersion.put(Superclasses, FindMedoid(PathMatrix));

		}

		return superClassDispersion;

	}

	public double CalculateDisparity(Hashtable<Object, Float> superClassDispersion) {

		float meanSum = 0;
		Enumeration en1 = superClassDispersion.keys();
		while (en1.hasMoreElements()) {
			Object superclass1 = en1.nextElement();

			float dispersion1 = superClassDispersion.get(superclass1);
			meanSum = meanSum + dispersion1;
		}

		double disparityBH = meanSum / superClassDispersion.size();
		return disparityBH;
	}

	private float FindMedoid(Hashtable<Object, List<Integer>> PathMatrix) {
		Object medoid = null;
		float min;
		float disparity = 0;

		Hashtable<Object, Float> average = new Hashtable<Object, Float>();

		// calculate average (midoid is the element with the minimal average distances//
		// the closest entity to all other entities
		Enumeration e = PathMatrix.keys();
		while (e.hasMoreElements()) {
			int sum = 0;
			float avg = 0;
			Object tag = e.nextElement();
			List<Integer> shortestPath = PathMatrix.get(tag);
			for (int i = 0; i < shortestPath.size(); i++) {
				sum = sum + shortestPath.get(i);
			}
			avg = (float) sum / shortestPath.size();

			// put uri and average distances (average of shortest path for this entity with
			// all other entities)
			average.put(tag, avg);
		}

		// Find medoid, minimum average

		Enumeration e1 = average.keys();
		Object tag1 = e1.nextElement();
		min = average.get(tag1);

		while (e1.hasMoreElements()) {

			Object tag2 = e1.nextElement();
			float ave = average.get(tag2);
			if (average.get(tag2) < min) {
				min = average.get(tag2);
			}
		}

		for (Map.Entry<Object, Float> entry : average.entrySet()) {
			if (entry.getValue() == min) {
				medoid = entry.getKey();
			}
		}

		if (medoid.equals(null)) {
			System.out.println("Oops no mediod is found");
		} else {

		}
		// Now, identify which element is the medoid and save its distances

		Enumeration e3 = PathMatrix.keys();
		while (e3.hasMoreElements()) {
			Object tag4 = e3.nextElement();
			List<Integer> distanceValues = PathMatrix.get(tag4);
			if (tag4.equals(medoid)) {
				disparity = BallHallDespersion(distanceValues, PathMatrix.size());
			}
		}
		return disparity;
	}

	private float BallHallDespersion(List<Integer> distanceValues, int superclassSize) {

		int sum = 0;
		float sum1 = 0;
		float dispersion = 0;

		// find mean (dispersion) from Ball-Hall for each cluster/super class
		for (int i = 0; i < distanceValues.size(); i++) {
			sum = sum + (distanceValues.get(i) * distanceValues.get(i));
		}
		dispersion = (float) sum / superclassSize;

		return dispersion;
	}

	public static Object FindParentClass(Object c, OntModel model, Object superclass) {
		Individual d = model.getIndividual(c.toString());
		OntClass individualParentClass = d.getOntClass();
		Resource Superclass = model.getResource(superclass.toString());

		if (individualParentClass.hasSuperClass(Superclass)) {

			return individualParentClass;
		} else {
			return CheckParentClass(d, model, superclass);
		}
	}

	public static Object CheckParentClass(Individual d, OntModel model, Object superclass) {

		OntClass SC = null;

		OntClass Sclass = model.getOntClass(superclass.toString());

		for (ExtendedIterator<? extends OntResource> j = Sclass.listInstances(); j.hasNext();) {
			Object instance1 = j.next();
			if (instance1.equals(d)) {
				SC = Sclass;
				break;

			}
		}
		if (SC == null) {
			for (ExtendedIterator<OntClass> k = Sclass.listSubClasses(false); k.hasNext();) {
				OntClass cls = k.next();

				for (Iterator i = cls.listInstances(true); i.hasNext();) {
					Object instance2 = i.next();
					if (instance2.equals(d)) {
						SC = cls;
						break;

					}

				}
			}

		}
		if (SC.equals(null)) {
			System.out.println("I am an instance cannot find my parent class");
		}
		return SC;

	}

}
